/*
 * Copyright (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICEutil.h>
#include <X11/SM/SMlib.h>
#include <X11/Xlib.h>

#include <libxfce4util/i18n.h>
#include <libxfce4util/util.h>
#include <libxfcegui4/libxfcegui4.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include "client.h"
#include "ice-layer.h"
#include "manager.h"
#include "pending.h"
#include "splash-icon.h"
#include "util.h"

/* list of restored, but not yet reconnected SM clients */
GList	*pendingClients = NULL;

/* */
static guint	pendingTimeoutId = 0;

/* */
#define MOE_LOGOS	5
static GdkPixbuf	*moeLogos[MOE_LOGOS];
static guint		moeLogo = 0;

/* */
static GtkWidget	*splashWindow = NULL;
static GtkWidget	*splashProgress;
static GtkWidget	*splashLogo;

/* */
static double		clientsToStart;
static double		clientsStarted;

/* */
#define PENDING_TIMEOUT		(5 * 1000)

/*
 */
static gboolean
pending_timeout(Client *client)
{
	/* client startup timed out */
	if (client != NULL)
		g_warning("client %s timed out", client->id);
	pending_continue(client);
	return(FALSE);
}

/*
 */
static gboolean
pending_launch(void)
{
	Client *client;
	GError *error;
	gchar *program;
	gchar *buffer;

	for (;;) {
		/* REQUIRED! */
		error = NULL;

		/* check if we are finished */
		if (g_list_length(pendingClients) == 0) {
			gtk_widget_destroy(splashWindow);
			break;
		}

		/* get next client to fire up */
		client = CLIENT(g_list_first(pendingClients)->data);

		/* try to start client */
		if (!client_run_command(client, SmRestartCommand, &error)) {
			g_warning("Unable to start client %s: %s", client->id,
					error->message);
			g_error_free(error);

			/* run DiscardCommand */
			client_run_command(client, SmDiscardCommand, NULL);

			/* remove client */
			pendingClients = g_list_remove(pendingClients, client);
			client_free(client);

			/* try next client */
			continue;
		}

		/* notify the user of what we are doing */
		if ((program = client_get_program(client)) != NULL) {
			buffer = g_strdup_printf(_("Starting %s..."), program);
			gtk_progress_bar_set_text(
				GTK_PROGRESS_BAR(splashProgress), buffer);
			g_free(buffer);
			g_free(program);
		}

		/* add startup timeout */
		pendingTimeoutId = g_timeout_add(PENDING_TIMEOUT,
				(GSourceFunc)pending_timeout, client);
		break;
	}

	return(FALSE);
}

/*
 */
void
pending_continue(Client *client)
{
	if (pendingTimeoutId != 0)
		g_source_remove(pendingTimeoutId);

	if (client != NULL) {
		pendingClients = g_list_remove(pendingClients, client);
		client_free(client);
	}
	else if (splashWindow == NULL) {
		GtkWidget *vbox;

		moeLogos[0] = inline_icon_at_size(splash_icon_data1, 350, 350);
		moeLogos[1] = inline_icon_at_size(splash_icon_data2, 350, 350);
		moeLogos[2] = inline_icon_at_size(splash_icon_data3, 350, 350);
		moeLogos[3] = inline_icon_at_size(splash_icon_data4, 350, 350);
		moeLogos[4] = inline_icon_at_size(splash_icon_data5, 350, 350);

		/* Ok, start up the splash screen */
		splashWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_position(GTK_WINDOW(splashWindow),
				GTK_WIN_POS_CENTER_ALWAYS);
		gtk_window_set_decorated(GTK_WINDOW(splashWindow), FALSE);
		gtk_window_stick(GTK_WINDOW(splashWindow));

		/* */
		netk_gtk_window_set_type(GTK_WINDOW(splashWindow),
				NETK_WINDOW_SPLASHSCREEN);

		/* use a watch cursor */
		gtk_widget_realize(splashWindow);
		gdk_window_set_cursor(splashWindow->window,
				gdk_cursor_new(GDK_WATCH));

		/* */
		vbox = gtk_vbox_new(FALSE, 1);
		gtk_container_add(GTK_CONTAINER(splashWindow), vbox);

		/* the logo */
		splashLogo = gtk_image_new_from_pixbuf(moeLogos[moeLogo]);
		gtk_box_pack_start(GTK_BOX(vbox), splashLogo, TRUE, TRUE, 0);

		/* the progress bar */
		splashProgress = gtk_progress_bar_new();
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(splashProgress),
				_("Starting session manager..."));
		gtk_box_pack_start(GTK_BOX(vbox), splashProgress, FALSE,
				TRUE, 0);

		gtk_widget_show_all(splashWindow);

		clientsToStart = g_list_length(pendingClients);
		clientsStarted = 0.0;

		/* XXX - give time to the splash screen to appear */
		g_idle_add((GSourceFunc)pending_timeout, NULL);
		return;
	}

	/* just another client started */
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(splashProgress),
			clientsStarted++ / clientsToStart);

	moeLogo = (moeLogo + 1) % MOE_LOGOS;
	gtk_image_set_from_pixbuf(GTK_IMAGE(splashLogo), moeLogos[moeLogo]);

	g_idle_add((GSourceFunc)pending_launch, NULL);
}


